@@ -15,9 +15,7 @@
 # You should have received a copy of the GNU General Public License
 # along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
 
-# Make coding more python3-ish
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+from __future__ import annotations
 
 import cmd
 import functools
@@ -27,6 +25,7 @@ import queue
 import sys
 import threading
 import time
+import typing as t
 
 from collections import deque
 from multiprocessing import Lock
@@ -37,12 +36,12 @@ from ansible import constants as C
 from ansible import context
 from ansible.errors import AnsibleError, AnsibleFileNotFound, AnsibleUndefinedVariable, AnsibleParserError
 from ansible.executor import action_write_locks
-from ansible.executor.play_iterator import IteratingStates
+from ansible.executor.play_iterator import IteratingStates, PlayIterator
 from ansible.executor.process.worker import WorkerProcess
 from ansible.executor.task_result import TaskResult
-from ansible.executor.task_queue_manager import CallbackSend, DisplaySend
+from ansible.executor.task_queue_manager import CallbackSend, DisplaySend, PromptSend
 from ansible.module_utils.six import string_types
-from ansible.module_utils._text import to_text
+from ansible.module_utils.common.text.converters import to_text
 from ansible.module_utils.connection import Connection, ConnectionError
 from ansible.playbook.conditional import Conditional
 from ansible.playbook.handler import Handler
@@ -54,7 +53,8 @@ from ansible.template import Templar
 from ansible.utils.display import Display
 from ansible.utils.fqcn import add_internal_fqcns
 from ansible.utils.unsafe_proxy import wrap_var
-from ansible.utils.vars import combine_vars, isidentifier
+from ansible.utils.sentinel import Sentinel
+from ansible.utils.vars import combine_vars
 from ansible.vars.clean import strip_internal_keys, module_response_deepcopy
 
 display = Display()
@@ -115,7 +115,8 @@ def results_thread_main(strategy):
             if isinstance(result, StrategySentinel):
                 break
             elif isinstance(result, DisplaySend):
-                display.display(*result.args, **result.kwargs)
+                dmethod = getattr(display, result.method)
+                dmethod(*result.args, **result.kwargs)
             elif isinstance(result, CallbackSend):
                 for arg in result.args:
                     if isinstance(arg, TaskResult):
@@ -126,6 +127,24 @@ def results_thread_main(strategy):
                 strategy.normalize_task_result(result)
                 with strategy._results_lock:
                     strategy._results.append(result)
+            elif isinstance(result, PromptSend):
+                try:
+                    value = display.prompt_until(
+                        result.prompt,
+                        private=result.private,
+                        seconds=result.seconds,
+                        complete_input=result.complete_input,
+                        interrupt_input=result.interrupt_input,
+                    )
+                except AnsibleError as e:
+                    value = e
+                except BaseException as e:
+                    # relay unexpected errors so bugs in display are reported and don't cause workers to hang
+                    try:
+                        raise AnsibleError(f"{e}") from e
+                    except AnsibleError as e:
+                        value = e
+                strategy._workers[result.worker_id].worker_queue.put(value)
             else:
                 display.warning('Received an invalid object (%s) in the result queue: %r' % (type(result), result))
         except (IOError, EOFError):
@@ -242,6 +261,8 @@ class StrategyBase:
         self._results = deque()
         self._results_lock = threading.Condition(threading.Lock())
 
+        self._worker_queues = dict()
+
         # create the result processing thread for reading results in the background
         self._results_thread = threading.Thread(target=results_thread_main, args=(self,))
         self._results_thread.daemon = True
@@ -385,7 +406,10 @@ class StrategyBase:
                         'play_context': play_context
                     }
 
-                    worker_prc = WorkerProcess(self._final_q, task_vars, host, task, play_context, self._loader, self._variable_manager, plugin_loader)
+                    # Pass WorkerProcess its strategy worker number so it can send an identifier along with intra-task requests
+                    worker_prc = WorkerProcess(
+                        self._final_q, task_vars, host, task, play_context, self._loader, self._variable_manager, plugin_loader, self._cur_worker,
+                    )
                     self._workers[self._cur_worker] = worker_prc
                     self._tqm.send_callback('v2_runner_on_start', host, task)
                     worker_prc.start()
@@ -482,56 +506,64 @@ class StrategyBase:
 
         return task_result
 
+    def search_handlers_by_notification(self, notification: str, iterator: PlayIterator) -> t.Generator[Handler, None, None]:
+        templar = Templar(None)
+        handlers = [h for b in reversed(iterator._play.handlers) for h in b.block]
+        # iterate in reversed order since last handler loaded with the same name wins
+        for handler in handlers:
+            if not handler.name:
+                continue
+            if not handler.cached_name:
+                if templar.is_template(handler.name):
+                    templar.available_variables = self._variable_manager.get_vars(
+                        play=iterator._play,
+                        task=handler,
+                        _hosts=self._hosts_cache,
+                        _hosts_all=self._hosts_cache_all
+                    )
+                    try:
+                        handler.name = templar.template(handler.name)
+                    except (UndefinedError, AnsibleUndefinedVariable) as e:
+                        # We skip this handler due to the fact that it may be using
+                        # a variable in the name that was conditionally included via
+                        # set_fact or some other method, and we don't want to error
+                        # out unnecessarily
+                        if not handler.listen:
+                            display.warning(
+                                "Handler '%s' is unusable because it has no listen topics and "
+                                "the name could not be templated (host-specific variables are "
+                                "not supported in handler names). The error: %s" % (handler.name, to_text(e))
+                            )
+                        continue
+                handler.cached_name = True
+
+            # first we check with the full result of get_name(), which may
+            # include the role name (if the handler is from a role). If that
+            # is not found, we resort to the simple name field, which doesn't
+            # have anything extra added to it.
+            if notification in {
+                handler.name,
+                handler.get_name(include_role_fqcn=False),
+                handler.get_name(include_role_fqcn=True),
+            }:
+                yield handler
+                break
+
+        seen = set()
+        for handler in handlers:
+            if notification in handler.listen:
+                if handler.name and handler.name in seen:
+                    continue
+                seen.add(handler.name)
+                yield handler
+
     @debug_closure
     def _process_pending_results(self, iterator, one_pass=False, max_passes=None):
         '''
         Reads results off the final queue and takes appropriate action
         based on the result (executing callbacks, updating state, etc.).
         '''
-
         ret_results = []
-        handler_templar = Templar(self._loader)
-
-        def search_handler_blocks_by_name(handler_name, handler_blocks):
-            # iterate in reversed order since last handler loaded with the same name wins
-            for handler_block in reversed(handler_blocks):
-                for handler_task in handler_block.block:
-                    if handler_task.name:
-                        try:
-                            if not handler_task.cached_name:
-                                if handler_templar.is_template(handler_task.name):
-                                    handler_templar.available_variables = self._variable_manager.get_vars(play=iterator._play,
-                                                                                                          task=handler_task,
-                                                                                                          _hosts=self._hosts_cache,
-                                                                                                          _hosts_all=self._hosts_cache_all)
-                                    handler_task.name = handler_templar.template(handler_task.name)
-                                handler_task.cached_name = True
-
-                            # first we check with the full result of get_name(), which may
-                            # include the role name (if the handler is from a role). If that
-                            # is not found, we resort to the simple name field, which doesn't
-                            # have anything extra added to it.
-                            candidates = (
-                                handler_task.name,
-                                handler_task.get_name(include_role_fqcn=False),
-                                handler_task.get_name(include_role_fqcn=True),
-                            )
-
-                            if handler_name in candidates:
-                                return handler_task
-                        except (UndefinedError, AnsibleUndefinedVariable) as e:
-                            # We skip this handler due to the fact that it may be using
-                            # a variable in the name that was conditionally included via
-                            # set_fact or some other method, and we don't want to error
-                            # out unnecessarily
-                            if not handler_task.listen:
-                                display.warning(
-                                    "Handler '%s' is unusable because it has no listen topics and "
-                                    "the name could not be templated (host-specific variables are "
-                                    "not supported in handler names). The error: %s" % (handler_task.name, to_text(e))
-                                )
-                            continue
-
         cur_pass = 0
         while True:
             try:
@@ -562,7 +594,7 @@ class StrategyBase:
                     else:
                         iterator.mark_host_failed(original_host)
 
-                    state, _ = iterator.get_next_task_for_host(original_host, peek=True)
+                    state, dummy = iterator.get_next_task_for_host(original_host, peek=True)
 
                     if iterator.is_failed(original_host) and state and state.run_state == IteratingStates.COMPLETE:
                         self._tqm._failed_hosts[original_host.name] = True
@@ -612,49 +644,33 @@ class StrategyBase:
                     result_items = [task_result._result]
 
                 for result_item in result_items:
-                    if '_ansible_notify' in result_item:
-                        if task_result.is_changed():
-                            # The shared dictionary for notified handlers is a proxy, which
-                            # does not detect when sub-objects within the proxy are modified.
-                            # So, per the docs, we reassign the list so the proxy picks up and
-                            # notifies all other threads
-                            for handler_name in result_item['_ansible_notify']:
-                                found = False
-                                # Find the handler using the above helper.  First we look up the
-                                # dependency chain of the current task (if it's from a role), otherwise
-                                # we just look through the list of handlers in the current play/all
-                                # roles and use the first one that matches the notify name
-                                target_handler = search_handler_blocks_by_name(handler_name, iterator._play.handlers)
-                                if target_handler is not None:
-                                    found = True
-                                    if target_handler.notify_host(original_host):
-                                        self._tqm.send_callback('v2_playbook_on_notify', target_handler, original_host)
-
-                                for listening_handler_block in iterator._play.handlers:
-                                    for listening_handler in listening_handler_block.block:
-                                        listeners = getattr(listening_handler, 'listen', []) or []
-                                        if not listeners:
-                                            continue
-
-                                        listeners = listening_handler.get_validated_value(
-                                            'listen', listening_handler.fattributes.get('listen'), listeners, handler_templar
-                                        )
-                                        if handler_name not in listeners:
-                                            continue
-                                        else:
-                                            found = True
-
-                                        if listening_handler.notify_host(original_host):
-                                            self._tqm.send_callback('v2_playbook_on_notify', listening_handler, original_host)
-
-                                # and if none were found, then we raise an error
-                                if not found:
-                                    msg = ("The requested handler '%s' was not found in either the main handlers list nor in the listening "
-                                           "handlers list" % handler_name)
-                                    if C.ERROR_ON_MISSING_HANDLER:
-                                        raise AnsibleError(msg)
-                                    else:
-                                        display.warning(msg)
+                    if '_ansible_notify' in result_item and task_result.is_changed():
+                        # only ensure that notified handlers exist, if so save the notifications for when
+                        # handlers are actually flushed so the last defined handlers are executed,
+                        # otherwise depending on the setting either error or warn
+                        host_state = iterator.get_state_for_host(original_host.name)
+                        for notification in result_item['_ansible_notify']:
+                            handler = Sentinel
+                            for handler in self.search_handlers_by_notification(notification, iterator):
+                                if host_state.run_state == IteratingStates.HANDLERS:
+                                    # we're currently iterating handlers, so we need to expand this now
+                                    if handler.notify_host(original_host):
+                                        # NOTE even with notifications deduplicated this can still happen in case of handlers being
+                                        # notified multiple times using different names, like role name or fqcn
+                                        self._tqm.send_callback('v2_playbook_on_notify', handler, original_host)
+                                else:
+                                    iterator.add_notification(original_host.name, notification)
+                                    display.vv(f"Notification for handler {notification} has been saved.")
+                                    break
+                            if handler is Sentinel:
+                                msg = (
+                                    f"The requested handler '{notification}' was not found in either the main handlers"
+                                    " list nor in the listening handlers list"
+                                )
+                                if C.ERROR_ON_MISSING_HANDLER:
+                                    raise AnsibleError(msg)
+                                else:
+                                    display.warning(msg)
 
                     if 'add_host' in result_item:
                         # this task added a new host (add_host module)
@@ -676,7 +692,7 @@ class StrategyBase:
                         else:
                             all_task_vars = found_task_vars
                         all_task_vars[original_task.register] = wrap_var(result_item)
-                        post_process_whens(result_item, original_task, handler_templar, all_task_vars)
+                        post_process_whens(result_item, original_task, Templar(self._loader), all_task_vars)
                         if original_task.loop or original_task.loop_with:
                             new_item_result = TaskResult(
                                 task_result._host,
@@ -750,10 +766,6 @@ class StrategyBase:
 
             # register final results
             if original_task.register:
-
-                if not isidentifier(original_task.register):
-                    raise AnsibleError("Invalid variable name in 'register' specified: '%s'" % original_task.register)
-
                 host_list = self.get_task_hosts(iterator, original_host, original_task)
 
                 clean_copy = strip_internal_keys(module_response_deepcopy(task_result._result))
@@ -770,18 +782,13 @@ class StrategyBase:
             # If this is a role task, mark the parent role as being run (if
             # the task was ok or failed, but not skipped or unreachable)
             if original_task._role is not None and role_ran:  # TODO:  and original_task.action not in C._ACTION_INCLUDE_ROLE:?
-                # lookup the role in the ROLE_CACHE to make sure we're dealing
+                # lookup the role in the role cache to make sure we're dealing
                 # with the correct object and mark it as executed
-                for (entry, role_obj) in iterator._play.ROLE_CACHE[original_task._role.get_name()].items():
-                    if role_obj._uuid == original_task._role._uuid:
-                        role_obj._had_task_run[original_host.name] = True
+                role_obj = self._get_cached_role(original_task, iterator._play)
+                role_obj._had_task_run[original_host.name] = True
 
             ret_results.append(task_result)
 
-            if isinstance(original_task, Handler):
-                for handler in (h for b in iterator._play.handlers for h in b.block if h._uuid == original_task._uuid):
-                    handler.remove_host(original_host)
-
             if one_pass or max_passes is not None and (cur_pass + 1) >= max_passes:
                 break
 
@@ -825,7 +832,7 @@ class StrategyBase:
 
         return ti_copy
 
-    def _load_included_file(self, included_file, iterator, is_handler=False):
+    def _load_included_file(self, included_file, iterator, is_handler=False, handle_stats_and_callbacks=True):
         '''
         Loads an included YAML file of tasks, applying the optional set of variables.
 
@@ -833,6 +840,15 @@ class StrategyBase:
         in such case the caller is responsible for marking the host(s) as failed
         using PlayIterator.mark_host_failed().
         '''
+        if handle_stats_and_callbacks:
+            display.deprecated(
+                "Reporting play recap stats and running callbacks functionality for "
+                "``include_tasks`` in ``StrategyBase._load_included_file`` is deprecated. "
+                "See ``https://github.com/ansible/ansible/pull/79260`` for guidance on how to "
+                "move the reporting into specific strategy plugins to account for "
+                "``include_role`` tasks as well.",
+                version="2.21"
+            )
         display.debug("loading included file: %s" % included_file._filename)
         try:
             data = self._loader.load_from_file(included_file._filename)
@@ -852,11 +868,9 @@ class StrategyBase:
                 loader=self._loader,
                 variable_manager=self._variable_manager,
             )
-
-            # since we skip incrementing the stats when the task result is
-            # first processed, we do so now for each host in the list
-            for host in included_file._hosts:
-                self._tqm._stats.increment('ok', host.name)
+            if handle_stats_and_callbacks:
+                for host in included_file._hosts:
+                    self._tqm._stats.increment('ok', host.name)
         except AnsibleParserError:
             raise
         except AnsibleError as e:
@@ -864,18 +878,18 @@ class StrategyBase:
                 reason = "Could not find or access '%s' on the Ansible Controller." % to_text(e.file_name)
             else:
                 reason = to_text(e)
+            if handle_stats_and_callbacks:
+                for r in included_file._results:
+                    r._result['failed'] = True
 
-            for r in included_file._results:
-                r._result['failed'] = True
-
-            for host in included_file._hosts:
-                tr = TaskResult(host=host, task=included_file._task, return_data=dict(failed=True, reason=reason))
-                self._tqm._stats.increment('failures', host.name)
-                self._tqm.send_callback('v2_runner_on_failed', tr)
+                for host in included_file._hosts:
+                    tr = TaskResult(host=host, task=included_file._task, return_data=dict(failed=True, reason=reason))
+                    self._tqm._stats.increment('failures', host.name)
+                    self._tqm.send_callback('v2_runner_on_failed', tr)
             raise AnsibleError(reason) from e
 
-        # finally, send the callback and return the list of blocks loaded
-        self._tqm.send_callback('v2_playbook_on_include', included_file)
+        if handle_stats_and_callbacks:
+            self._tqm.send_callback('v2_playbook_on_include', included_file)
         display.debug("done processing included file")
         return block_list
 
@@ -912,6 +926,8 @@ class StrategyBase:
         meta_action = task.args.get('_raw_params')
 
         def _evaluate_conditional(h):
+            if not task.when:
+                return True
             all_vars = self._variable_manager.get_vars(play=iterator._play, host=h, task=task,
                                                        _hosts=self._hosts_cache, _hosts_all=self._hosts_cache_all)
             templar = Templar(loader=self._loader, variables=all_vars)
@@ -934,6 +950,15 @@ class StrategyBase:
         elif meta_action == 'flush_handlers':
             if _evaluate_conditional(target_host):
                 host_state = iterator.get_state_for_host(target_host.name)
+                # actually notify proper handlers based on all notifications up to this point
+                for notification in list(host_state.handler_notifications):
+                    for handler in self.search_handlers_by_notification(notification, iterator):
+                        if handler.notify_host(target_host):
+                            # NOTE even with notifications deduplicated this can still happen in case of handlers being
+                            # notified multiple times using different names, like role name or fqcn
+                            self._tqm.send_callback('v2_playbook_on_notify', handler, target_host)
+                    iterator.clear_notification(target_host.name, notification)
+
                 if host_state.run_state == IteratingStates.HANDLERS:
                     raise AnsibleError('flush_handlers cannot be used as a handler')
                 if target_host.name not in self._tqm._unreachable_hosts:
@@ -970,7 +995,7 @@ class StrategyBase:
             if _evaluate_conditional(target_host):
                 for host in self._inventory.get_hosts(iterator._play.hosts):
                     if host.name not in self._tqm._unreachable_hosts:
-                        iterator.set_run_state_for_host(host.name, IteratingStates.COMPLETE)
+                        iterator.end_host(host.name)
                 msg = "ending batch"
             else:
                 skipped = True
@@ -979,7 +1004,7 @@ class StrategyBase:
             if _evaluate_conditional(target_host):
                 for host in self._inventory.get_hosts(iterator._play.hosts):
                     if host.name not in self._tqm._unreachable_hosts:
-                        iterator.set_run_state_for_host(host.name, IteratingStates.COMPLETE)
+                        iterator.end_host(host.name)
                         # end_play is used in PlaybookExecutor/TQM to indicate that
                         # the whole play is supposed to be ended as opposed to just a batch
                         iterator.end_play = True
@@ -989,8 +1014,7 @@ class StrategyBase:
                 skip_reason += ', continuing play'
         elif meta_action == 'end_host':
             if _evaluate_conditional(target_host):
-                iterator.set_run_state_for_host(target_host.name, IteratingStates.COMPLETE)
-                iterator._play._removed_hosts.append(target_host.name)
+                iterator.end_host(target_host.name)
                 msg = "ending play for %s" % target_host.name
             else:
                 skipped = True
@@ -998,12 +1022,23 @@ class StrategyBase:
                 # TODO: Nix msg here? Left for historical reasons, but skip_reason exists now.
                 msg = "end_host conditional evaluated to false, continuing execution for %s" % target_host.name
         elif meta_action == 'role_complete':
-            # Allow users to use this in a play as reported in https://github.com/ansible/ansible/issues/22286?
-            # How would this work with allow_duplicates??
             if task.implicit:
-                if target_host.name in task._role._had_task_run:
-                    task._role._completed[target_host.name] = True
+                role_obj = self._get_cached_role(task, iterator._play)
+                if target_host.name in role_obj._had_task_run:
+                    role_obj._completed[target_host.name] = True
                     msg = 'role_complete for %s' % target_host.name
+        elif meta_action == 'end_role':
+            if _evaluate_conditional(target_host):
+                while True:
+                    state, task = iterator.get_next_task_for_host(target_host, peek=True)
+                    if task.action in C._ACTION_META and task.args.get("_raw_params") == "role_complete":
+                        break
+                    iterator.set_state_for_host(target_host.name, state)
+                    display.debug("'%s' skipped because role has been ended via 'end_role'" % task)
+                msg = 'ending role %s for %s' % (task._role.get_name(), target_host.name)
+            else:
+                skipped = True
+                skip_reason += 'continuing role %s for %s' % (task._role.get_name(), target_host.name)
         elif meta_action == 'reset_connection':
             all_vars = self._variable_manager.get_vars(play=iterator._play, host=target_host, task=task,
                                                        _hosts=self._hosts_cache, _hosts_all=self._hosts_cache_all)
@@ -1059,14 +1094,20 @@ class StrategyBase:
             header = skip_reason if skipped else msg
             display.vv(f"META: {header}")
 
-        if isinstance(task, Handler):
-            task.remove_host(target_host)
-
         res = TaskResult(target_host, task, result)
         if skipped:
             self._tqm.send_callback('v2_runner_on_skipped', res)
         return [res]
 
+    def _get_cached_role(self, task, play):
+        role_path = task._role.get_role_path()
+        role_cache = play.role_cache[role_path]
+        try:
+            idx = role_cache.index(task._role)
+            return role_cache[idx]
+        except ValueError:
+            raise AnsibleError(f'Cannot locate {task._role.get_name()} in role cache')
+
     def get_hosts_left(self, iterator):
         ''' returns list of available hosts for this iterator by filtering out unreachables '''
 
