@@ -1,6 +1,5 @@
 #!/usr/bin/python
-from __future__ import absolute_import, division, print_function
-__metaclass__ = type
+from __future__ import annotations
 
 
 ANSIBLE_METADATA = {'metadata_version': '1.1',
