@@ -49,7 +49,7 @@ DOCKER_COMMANDS = [
     'podman',
 ]
 
-UTILITY_IMAGE = 'quay.io/ansible/ansible-test-utility-container:2.0.0'
+UTILITY_IMAGE = 'quay.io/ansible/ansible-test-utility-container:3.1.0'
 
 # Max number of open files in a docker container.
 # Passed with --ulimit option to the docker run command.
@@ -302,7 +302,7 @@ def detect_host_properties(args: CommonConfig) -> ContainerHostProperties:
     options = ['--volume', '/sys/fs/cgroup:/probe:ro']
     cmd = ['sh', '-c', ' && echo "-" && '.join(multi_line_commands)]
 
-    stdout, stderr = run_utility_container(args, f'ansible-test-probe-{args.session_name}', cmd, options)
+    stdout, stderr = run_utility_container(args, 'ansible-test-probe', cmd, options)
 
     if args.explain:
         return ContainerHostProperties(
@@ -344,7 +344,7 @@ def detect_host_properties(args: CommonConfig) -> ContainerHostProperties:
         cmd = ['sh', '-c', 'ulimit -Hn']
 
         try:
-            stdout = run_utility_container(args, f'ansible-test-ulimit-{args.session_name}', cmd, options)[0]
+            stdout = run_utility_container(args, 'ansible-test-ulimit', cmd, options)[0]
         except SubprocessError as ex:
             display.warning(str(ex))
         else:
@@ -410,6 +410,11 @@ def detect_host_properties(args: CommonConfig) -> ContainerHostProperties:
     return properties
 
 
+def get_session_container_name(args: CommonConfig, name: str) -> str:
+    """Return the given container name with the current test session name applied to it."""
+    return f'{name}-{args.session_name}'
+
+
 def run_utility_container(
     args: CommonConfig,
     name: str,
@@ -418,6 +423,8 @@ def run_utility_container(
     data: t.Optional[str] = None,
 ) -> tuple[t.Optional[str], t.Optional[str]]:
     """Run the specified command using the ansible-test utility container, returning stdout and stderr."""
+    name = get_session_container_name(args, name)
+
     options = options + [
         '--name', name,
         '--rm',
@@ -497,7 +504,7 @@ def get_docker_hostname() -> str:
     """Return the hostname of the Docker service."""
     docker_host = os.environ.get('DOCKER_HOST')
 
-    if docker_host and docker_host.startswith('tcp://'):
+    if docker_host and docker_host.startswith(('tcp://', 'ssh://')):
         try:
             hostname = urllib.parse.urlparse(docker_host)[1].split(':')[0]
             display.info('Detected Docker host: %s' % hostname, verbosity=1)
