@@ -1,6 +1,5 @@
 """A tool for probing cgroups to determine write access."""
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+from __future__ import annotations
 
 import json
 import os
