@@ -4,13 +4,12 @@
 # (c) 2017 Ansible Project
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-from __future__ import absolute_import, division, print_function
-__metaclass__ = type
+from __future__ import annotations
 
 from units.mock.procenv import ModuleTestCase
 
-from units.compat.mock import patch
-from ansible.module_utils.six.moves import builtins
+from unittest.mock import patch
+import builtins
 
 realimport = builtins.__import__
 
